#define F_CPU 16000000UL
#define BAUD 9600UL

#include <asf.h>
#include <avr/io.h>
#include <util/delay.h>
#include <util/setbaud.h>
#include <string.h>

#define cs_set_low PORTB &= ~(1<<DDB2);
#define a0_set_low PORTC &= ~(1<<DDC0);

void SPI_init(){
	DDRB = (1<<DDB3) | (1<<DDB5) | (1<<DDB2) | (1<<DDB0); //MOSI, SCK, SS, D8 as Output
	DDRB &= ~(1<<DDB4); //MISO as Input

	DDRD |= (1<<DDD2); //turn Vcc as Output to supply display

	SPCR = (1<<SPE) | (1<<MSTR); //init SPI & enable Master mode
	SPSR = (1<<SPI2X);//SCK = F_CPU/2, 
	_delay_ms(50);
}

void SPI_transmit(char s){
	SPDR = s; //start transmit
	while(!(SPSR & (1<<SPIF))) //transmit is done if SPIF is not one
	;
}

void reset_LCD(){
	PORTB &= ~(1<<DDB0); //turn Reset low
	_delay_ms(50);
	PORTB |= (1<<DDB0); //turn Reset high
	_delay_ms(120);
}

void write_command(uint8_t address){
	uint8_t i;
	PORTB &= ~(1<<DDB2); //turn CS low
	PORTC &= ~(1<<DDC0); //turn A0 low
	for(i = 0; i < 8; i++){
		if(address & 128){
			PORTB |= (1<<DDB3); //turn SDA high
		} else {
			PORTB &= ~(1<<DDB3); //turn SDA low
		}
		PORTB |= (1<<DDB5); //turn SCK high
		address <<=1;
		PORTB &= ~(1<<DDB5); //turn SCK low
	}
	_delay_ms(1);
	PORTB |= (1<<DDB2); //turn CS high
}

void write_parameter(uint8_t paramter){
	uint8_t i;

	PORTB &= ~(1<<DDB2); //turn CS low
	PORTC |= (1<<DDC0); //turn A0 high
	for(i = 0; i < 8; i++){
		if(paramter & 128){
			PORTB |= (1<<DDB3); //turn SDA high
		} else {
			PORTB &= ~(1<<DDB3); //turn SDA low
		}
		PORTB |= (1<<DDB5); //turn SCK high
		paramter <<= 1;
		PORTB &= ~(1<<DDB5); //turn SCK low
	}
	PORTB |= (1<<DDB2); //turn CS high
}

void writeData(uint16_t word){
	PORTB &= ~(1<<DDB2); //turn CS low
	PORTC |= (1<<DDC0); //turn A0 high
	for(int i = 0; i < 16; i++){
		if(word & 0x8000){
			PORTB |= (1<<DDB3); //turn SDA high
		} else {
			PORTB &= ~(1<<DDB3); //turn SDA low
		}
		PORTB |= (1<<DDB5); //write SCK high
		PORTB &= ~(1<<DDB5); //write SCK low
		word <<=1;
	}
	PORTB |= (1<<DDB2); //turn CS high
}

void init_LCD(){
	PORTB |= (1<<DDB2); //turn CS high
	PORTB &= ~(1<<DDB5); //turn SCK low
	PORTB |= (1<<DDB0); //turn RESET high

	reset_LCD();

	write_command(0x11); //exit sleep mode
	_delay_ms(5);

	write_command(0x3A); //set pixel format
	write_parameter(0x05); //16 bits per pixel

	write_command(0x26); //set Gamma curve
	write_parameter(0x04); //gamma curve 3

	write_command(0xF2); //gamma adjustment
	write_parameter(0x01); //enabled

	write_command(0xE0); //positive gamma correct
    write_parameter(0x3f); // 1st Parameter
    write_parameter(0x25); // 2nd Parameter
    write_parameter(0x1c); // 3rd Parameter
    write_parameter(0x1e); // 4th Parameter
    write_parameter(0x20); // 5th Parameter
    write_parameter(0x12); // 6th Parameter
    write_parameter(0x2a); // 7th Parameter
    write_parameter(0x90); // 8th Parameter
    write_parameter(0x24); // 9th Parameter
    write_parameter(0x11); // 10th Parameter
    write_parameter(0x00); // 11th Parameter
    write_parameter(0x00); // 12th Parameter
    write_parameter(0x00); // 13th Parameter
    write_parameter(0x00); // 14th Parameter
    write_parameter(0x00); // 15th Parameter

	write_command(0xE1); //negative gamma correct
    write_parameter(0x20); // 1st Parameter
    write_parameter(0x20); // 2nd Parameter
    write_parameter(0x20); // 3rd Parameter
    write_parameter(0x20); // 4th Parameter
    write_parameter(0x05); // 5th Parameter
    write_parameter(0x00); // 6th Parameter
    write_parameter(0x15); // 7th Parameter
    write_parameter(0xa7); // 8th Parameter
    write_parameter(0x3d); // 9th Parameter
    write_parameter(0x18); // 10th Parameter
    write_parameter(0x25); // 11th Parameter
    write_parameter(0x2a); // 12th Parameter
    write_parameter(0x2b); // 13th Parameter
    write_parameter(0x2b); // 14th Parameter
    write_parameter(0x3a); // 15th Parameter

	write_command(0xB1); //Frame rate control
	write_parameter(0x08); //DIVA=8
	write_parameter(0x08); //VPA=8

	write_command(0xB4);
	write_parameter(0x07);

	write_command(0xC0); //power control 1
	write_parameter(0x0a);
	write_parameter(0x02);

	write_command(0xC1); //power control 1
	write_parameter(0x02);

	write_command(0xC5); //VCOM control
	write_parameter(0x50);
	write_parameter(0x5b);

	write_command(0xC7); //VCOM offset control
	write_parameter(0x40);

	write_command(0x2A);
    write_parameter(0x00);
    write_parameter(0x00);
    write_parameter(0x00);
    write_parameter(0x7f);

	write_command(0x2B);
    write_parameter(0x00);
    write_parameter(0x00);
    write_parameter(0x00);
    write_parameter(0x7f);

	write_command(0x36);
	write_parameter(192); //orientation "flipped landscape" [0,96,160,192]

	write_command(0x29); //set display on
	write_command(0x2C);
}

void clear_LCD(int16_t color){
	uint16_t pixel;
	write_command(0x2A); //set column address
    write_parameter(0x00); // XSH
    write_parameter(0x00); // XSL
    write_parameter(0x00); // XEH
    write_parameter(0x7f); // XEL (128 pixels x)

	write_command(0x2B); //set page address
    write_parameter(0x00);
    write_parameter(0x00);
    write_parameter(0x00);
    write_parameter(0x7f); // 128 pixels y

	write_command(0x2C); //memory start
	for(pixel = 0; pixel < 16835; pixel++){
		writeData(color);
	}
}

int main (void)
{
	SPI_init();
	init_LCD();
	clear_LCD(0x07E0);
	DDRD |= (1<<DDD7); //turn D7 as output
	while(1){
		clear_LCD(0x07E0); //black
		_delay_ms(500);
	}
	return 0;
}
